import { ValidatorFn, AbstractControl } from '@angular/forms';

export function cpfValidator(): ValidatorFn {
    return (control: AbstractControl): {[key: string]: any} | null => {
        if( Validator.cpfIsInvalid(control.value)){
            return {cpfInvalido: control.value};
        }else{
            return null;
        }
    };
}

export function cnpjValidator(): ValidatorFn {
    return (control: AbstractControl): {[key: string]: any} | null => {
        if( Validator.cnpjIsInvalid(control.value)){
            return {cnpjInvalido: control.value};
        }else{
            return null;
        }
    };
}

export class Validator {
    public static readonly CPF_SIZE = 11;
    public static readonly PLACA_SIZE = 7;
    public static readonly CNPJ_SIZE = 14;
    public static readonly MAX_FILE_SIZE_5_MB = 5000000;

    static cpfIsInvalid(cpf: string): boolean {
        return !Validator.cpfIsValid(cpf);
    }

    static cpfIsValid(stringCPF: string): boolean {
        if (stringCPF === null || stringCPF === '') {
            return false;
        }
        if (stringCPF === '00000000000' ||
            stringCPF === '11111111111' ||
            stringCPF === '22222222222' ||
            stringCPF === '33333333333' ||
            stringCPF === '44444444444' ||
            stringCPF === '55555555555' ||
            stringCPF === '66666666666' ||
            stringCPF === '77777777777' ||
            stringCPF === '88888888888' ||
            stringCPF === '99999999999') {
            return false;
        }
        let sum = 0;
        for (let i = 1; i <= 9; i++) {
            sum += parseInt(stringCPF.substring(i - 1, i), 10) * (11 - i);
        }
        let remaining = (sum * 10) % 11;

        if (remaining === 10 || remaining === 11) {
            remaining = 0;
        }

        if (remaining !== parseInt(stringCPF.substring(9, 10), 10)) {
            return false;
        }

        sum = 0;

        for (let i = 1; i <= 10; i++) {
            sum += parseInt(stringCPF.substring(i - 1, i), 10) * (12 - i);
        }
        remaining = (sum * 10) % 11;

        if (remaining === 10 || remaining === 11) {
            remaining = 0;
        }

        if (remaining !== parseInt(stringCPF.substring(10, 11), 10)) {
            return false;
        }

        return true;
    }

    static cnpjIsInvalid(cnpj: string): boolean {
        return !Validator.cnpjIsValid(cnpj);
    }

    static cnpjIsValid(cnpj): boolean {

        if (cnpj === '' || cnpj === null) {
            return false;
        }
        if (cnpj.length !== 14) {
            return false;
        }
        if (cnpj === '00000000000000' ||
            cnpj === '11111111111111' ||
            cnpj === '22222222222222' ||
            cnpj === '33333333333333' ||
            cnpj === '44444444444444' ||
            cnpj === '55555555555555' ||
            cnpj === '66666666666666' ||
            cnpj === '77777777777777' ||
            cnpj === '88888888888888' ||
            cnpj === '99999999999999') {
            return false;
        }

        let size = cnpj.length - 2;
        let numbers = cnpj.substring(0, size);
        const digits = cnpj.substring(size);
        let sum = 0;
        let pos = size - 7;
        for (let i = size; i >= 1; i--) {
            sum += numbers.charAt(size - i) * pos--;
            if (pos < 2) {
                pos = 9;
            }
        }

        const primeiroDigitoVerificador = Number(digits.charAt(0));
        const segundoDigitoVerificador = Number(digits.charAt(1));
        let result = sum % 11 < 2 ? 0 : 11 - sum % 11;
        if (result !== primeiroDigitoVerificador) {
            return false;
        }
        size = size + 1;
        numbers = cnpj.substring(0, size);
        sum = 0;
        pos = size - 7;
        for (let i = size; i >= 1; i--) {
            sum += numbers.charAt(size - i) * pos--;
            if (pos < 2) {
                pos = 9;
            }
        }
        result = sum % 11 < 2 ? 0 : 11 - sum % 11;
        if (result !== segundoDigitoVerificador) {
            return false;
        }
        return true;
    }

    static cnpjFormatter(cnpj: string) {
        const cnpjFormated = cnpj.split('');
        cnpjFormated.splice(2, 0, '.');
        cnpjFormated.splice(6, 0, '.');
        cnpjFormated.splice(10, 0, '/');
        cnpjFormated.splice(15, 0, '-');
        const cnpjFormatedFinal = cnpjFormated.toString().replace(/[,]/g, '');
        return cnpjFormatedFinal;
    }

    static cpfFormatter(cpf: string): string {
        const cpfFormated = cpf.split('');
        cpfFormated.splice(3, 0, '.');
        cpfFormated.splice(7, 0, '.');
        cpfFormated.splice(11, 0, '-');
        const cpfFormatedFinal = cpfFormated.toString().replace(/[,]/g, '');
        return cpfFormatedFinal;
    }
}
